///////////////////////////////////////////////////////////////
//                                                           //
// dirNav.cpp is based on Dr. Fawcett's dir2.cpp             //
//                                                           //
// dirNav.cpp - traverse the directories under the current   //
//            directory. In each directory, invoke the       // 
//            function specified by the function pointer     //
//            in the argument list                           //
//                                                           //
///////////////////////////////////////////////////////////////
//                                                           //
// Adapted by Jim Fawcett from the book:                     //
// "Win32 System Services:                                   //
// The Heart of Windows 95 and Windows NT", Marshall Brain,  //
// Prentice Hall, Copyright 1995                             //
//                                                           //
///////////////////////////////////////////////////////////////
// Note: fixed bug in published code which fails to report   //
//       directory change to parent, associating files with  //
//       incorrect directory                                 //
///////////////////////////////////////////////////////////////

#include "dirNav.h"

//----< Recursively traverse directories >------------------------

void dirNav::traverse( const char * dirName, const char * fileMask )
{
  char *fileName;
  char curDir[ 256 ];
  char fullName[ 256 ];
  char prevFullName[256];
  HANDLE fileHandle;
  WIN32_FIND_DATA findData;
  

  // save current dir so it can restore it
  if( !GetCurrentDirectory( 256, curDir) ) 
    return;

  // if the directory name is neither . or .. then
  // change to it, otherwise ignore it
  if( strcmp( dirName, "." ) && strcmp( dirName, ".." ) ) {
	  if( !SetCurrentDirectory( dirName ) ) {
		  cerr << "Cannot find the directory \n" ;
		  return;
	  }
  } else {
    return;
  }

  // print out the current directory name
  strncpy(prevFullName,fullName,256);

  // if( !GetFullPathName( fileMask, 256, fullName, &fileName ) ) 
  if( !GetFullPathName( "." , 256, fullName, &fileName ) ) 
    return;

  // If the navigator gets the current directory name correctly,
  // it visit this directory
  // preProcessDir( curDir );
  preProcessDir( fullName );

  //if( strcmp( fullName,prevFullName ) != 0 )
  //  cout << "  " << fullName << endl;

  // Loop through all files in the directory
  fileHandle = FindFirstFile( fileMask, &findData );

  while ( fileHandle != INVALID_HANDLE_VALUE ) {
	  if( findData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ) {
		  // If it is a directory, traverse it recursively
		  // traverse( findData.cFileName, fileMask, autoDoForSubDir );
		  traverse( findData.cFileName, fileMask );
	  } else {
		  // Otherwise take the action on the file
		  // processFile( curDir,  findData.cFileName );
		  processFile( fullName,  findData.cFileName );
	  }

    // loop thru remaining entries in the dir
	  if (!FindNextFile( fileHandle, &findData ))
		  break;
  }

  postProcessDir( curDir );

  // clean up and restore directory
  FindClose( fileHandle );
  SetCurrentDirectory( curDir );

  // print out the current directory name
  // strncpy(prevFullName,fullName,256);
  // if( !GetFullPathName( fileMask, 256, fullName, &fileName ) ) 
  //   return;

  //if( strcmp( prevFullName,fullName ) != 0 )
  //  cout << "  " << fullName << endl;
}

//

//----< test stub >--------------------------------------------

#ifdef NAV_TEST

class testNav : public dirNav{
public:
	char preProcessDir( const char * dirName )
	{
		cout << "Entering directory: " << dirName << endl;
	}

	char postProcessDir( const char * dirName )
	{
		cout << "leaving directory: " << dirName << endl;
	}

	void performFile( const * dirName, const char * fName )
	{
		cout << "file name: " << fName << endl;
	}
};

int main(int argc, char *argv[])
{
  testNav * visitor = new testNav;
  cout << "\n  Demonstrating Directory Traversal "
       << "\n ===================================\n\n";

  char curDir[ 256 ];

  if( !GetCurrentDirectory( 256, curDir ) ) {
    cerr << "Couldn't get the current directory." << endl;
    return( 1 );
  }

  visitor->traverse( curDir, "*.*" );
  cout << endl;

  // List all files, starting with the current directory
  // visitor->traverse(curDir, "*.*", true);
  // cout << endl;

  return( 0 );
}

#endif